﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.DecisionList;
using HeuristicLab.Encodings.DecisionList.Interfaces;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Problems.DecisionListClassification {
  [Item("DecisionListClassificationProblem", "")]
  [StorableClass]
  [Creatable("Problems")]
  public class DecisionListClassificationProblem : HeuristicOptimizationProblem<IDecisionListEvaluator, IDecisionListCreator>, IDecisionListClassificationProblem {

    #region parameter properties
    public IFixedValueParameter<BoolValue> MaximizationParameter {
      get { return (IFixedValueParameter<BoolValue>)Parameters["Maximization"]; }
    }
    IParameter ISingleObjectiveHeuristicOptimizationProblem.MaximizationParameter {
      get { return MaximizationParameter; }
    }
    public IFixedValueParameter<DoubleValue> BestKnownQualityParameter {
      get { return (IFixedValueParameter<DoubleValue>)Parameters["BestKnownQuality"]; }
    }
    IParameter ISingleObjectiveHeuristicOptimizationProblem.BestKnownQualityParameter {
      get { return BestKnownQualityParameter; }
    }
    public IValueParameter<IDecisionListClassificationProblemData> ProblemDataParameter {
      get { return (IValueParameter<IDecisionListClassificationProblemData>)Parameters["ProblemData"]; }
    }
    public IFixedValueParameter<IntValue> SizePenaltyMinRulesParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters["SizePenaltyMinRules"]; }
    }
    public IFixedValueParameter<IntValue> ActivationIterationParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters["ActivationIteration"]; }
    }
    public IFixedValueParameter<DoubleValue> InitialTheoryLengthRatioParameter {
      get { return (IFixedValueParameter<DoubleValue>)Parameters["InitialTheoryLengthRatio"]; }
    }
    public IFixedValueParameter<DoubleValue> WeightRelaxFactorParameter {
      get { return (IFixedValueParameter<DoubleValue>)Parameters["WeightRelaxFactor"]; }
    }

    public IDecisionListClassificationProblemData ProblemData {
      get { return ProblemDataParameter.Value; }
      protected set {
        ProblemDataParameter.Value = value;
      }
    }
    IParameter IDecisionListClassificationProblem.ProblemDataParameter {
      get { return ProblemDataParameter; }
    }
    IDecisionListClassificationProblemData IDecisionListClassificationProblem.ProblemData {
      get { return ProblemData; }
    }

    IDecisionListEvaluator IDecisionListClassificationProblem.Evaluator {
      get { return Evaluator; }
    }
    ISingleObjectiveEvaluator ISingleObjectiveHeuristicOptimizationProblem.Evaluator {
      get { return Evaluator; }
    }
    #endregion

    [StorableConstructor]
    protected DecisionListClassificationProblem(bool deserializing) : base(deserializing) { }
    protected DecisionListClassificationProblem(DecisionListClassificationProblem original, Cloner cloner)
      : base(original, cloner) {
    }

    public DecisionListClassificationProblem()
      : this(new DecisionListClassificationProblemData(new Dataset(DecisionListClassificationProblemData.defaultVariableNames, DecisionListClassificationProblemData.defaultData),
        DecisionListClassificationProblemData.defaultVariableNames.Take(DecisionListClassificationProblemData.defaultVariableNames.Length - 1), DecisionListClassificationProblemData.defaultVariableNames.Last()),
        new MDLEvaluator(), new UniformRandomDecisionListCreator()) { }

    public DecisionListClassificationProblem(IDecisionListClassificationProblemData problemData, IDecisionListEvaluator decisionlistEvaluator, IDecisionListCreator decisionListCreator)
      : base(decisionlistEvaluator, decisionListCreator) {
      Parameters.Add(new ValueParameter<IDecisionListClassificationProblemData>("ProblemData", "", problemData));
      Parameters.Add(new FixedValueParameter<BoolValue>("Maximization", "", new BoolValue(true)));
      Parameters.Add(new FixedValueParameter<DoubleValue>("BestKnownQuality", "", new DoubleValue(0.5)));
      Parameters.Add(new FixedValueParameter<IntValue>("SizePenaltyMinRules", "", new IntValue(4)));
      Parameters.Add(new FixedValueParameter<IntValue>("ActivationIteration", "", new IntValue(25)));
      Parameters.Add(new FixedValueParameter<DoubleValue>("InitialTheoryLengthRatio", "", new DoubleValue(0.075)));
      Parameters.Add(new FixedValueParameter<DoubleValue>("WeightRelaxFactor", "", new DoubleValue(0.9)));

      Evaluator.SizePenaltyMinRulesParameter.ActualName = "SizePenaltyMinRules";
      // do differently
      ((MDLEvaluator)Evaluator).MDLCalculatorParameter.Value = new MDLCalculator(ActivationIterationParameter.Value.Value, InitialTheoryLengthRatioParameter.Value.Value, WeightRelaxFactorParameter.Value.Value);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DecisionListClassificationProblem(this, cloner);
    }
  }
}
